#include <qstring.h>
#include <qurl.h>
#include <qsocket.h>
#include <qapplication.h>

#include <unistd.h>
#include <iostream>
using namespace std;

#include "streaminput.h"
#include "shoutcaststream.h"
#include "mythtv/mythcontext.h"

ShoutcastStream::ShoutcastStream(QUrl *url)
                : QSocket()
{
    kconnected = 0;
    streamurl = *url;
    cout << "new shoutcast\n"; //DEBUG
}

bool ShoutcastStream::initialize()
{
    cout << "shoutcast: initialize\n"; //DEBUG
    if (!streamurl.isValid())
    {
        return false;
    }
    else
    {
        if (streamurl.protocol() == "shoutcast")
        {
            streamurl.setProtocol("http");
        }
        else
            return false;
        if (!streamurl.hasPort())
            streamurl.setPort(80);
    }
    return true;
}

bool ShoutcastStream::start()
{
    QString request;

    request = "GET " + streamurl.path().utf8() + " HTTP/1.0" + "\r\n";
    request += "Host: " + streamurl.host().utf8() + "\r\n";
    request += "User-Agent: xmms/1.2.7\r\n";
    request += "\r\n";

    cout << "shoutcast: start '" << request.ascii() << "'\n"; //DEBUG

    for (int i = 0; i < 3; i++)
    {
        if (openSocket())
            break;
    }
    if (state() != QSocket::Connected)
    {
        VERBOSE(VB_NETWORK, QString("Couldn't connect to: '%1'").arg(streamurl.host()));
        return false;
    }
    VERBOSE(VB_NETWORK, QString("Connected to: '%1'").arg(streamurl.host()));
    sendRequest(request);
    if (readlineSocket(request))
    {
        cout << request.ascii() << endl;
        if (request != "ICY 200 OK\r\n")
            return false;
        else
        {
            while (bytesAvailable() < 65536)
                usleep(100);
        }
    }
    else
        return false;

    kconnected = 1;
    return true ;
}

bool ShoutcastStream::openSocket()
{
    QString host = streamurl.host();
    int port = streamurl.port();
   
    qApp->lock();
    //socket = new QSocket();
    connectToHost(host, port);
    qApp->unlock();

    int i = 0;
    while (state() == QSocket::HostLookup ||
           state() == QSocket::Connecting)
    {
        qApp->processEvents();

        i++;
        if (i > 500)
            break;
        usleep(50);
    }
        
    if (state() != QSocket::Connected)
        return false;
    return true;
}

void ShoutcastStream::sendRequest(const QString request)
{
    writeBlock(request, request.length());
    flush();
    printf("\'%s\'",request.ascii()); //DEBUG
}

bool ShoutcastStream::readlineSocket(QString &request)
{
    int i = 0;
    while (state() == QSocket::Connected)
    {
        if (canReadLine())
        {
            request = readLine();
            return true;
        }
        i++;
        if (i > 500)
            break;
        usleep(50);
    }
    return false;
}

Q_LONG ShoutcastStream::readBlock(char *data, long unsigned maxlen)
{
    cout << "shoutcast: readBlock\n"; //DEBUG
    if (! kconnected)
    {
        cout << "shoutcast: readBlock N/C\n"; //DEBUG
        if (initialize())
        { 
            if (! start())
                return -1;
        }
        else
            return -1;
    }
    return QSocket::readBlock(data, maxlen);
}

